local UTILS = require("DZ_Utils")
local CONSTANTS = require("Constants")

local Climate = {module_name = "ClimateLore"}
Climate.__index = Climate

--------------------------------------------------------------------------------
function Climate:new()
    -- Keeps track and modifies the sandbox settings for the zombies
    local self = setmetatable({}, Climate)
    
    self.enable = false
    self.module_name = "Project Zomboid Climate" -- This is the human readable name for the module
    self.sandbox_prefix = "ClimateLore" -- This is the prefix used to query zombie lore from the sandbox settings
    
    --self.current_interval = 0 -- The current value of the climate

    self.intervals = 3
    self.interval_value = {}
    
    -- How much to modify the zombie lore by
    self.modifier_speed = 0
    self.modifier_strength = 0
    self.modifier_toughness = 0
    self.modifier_cognition = 0
    self.modifier_memory = 0
    self.modifier_sight = 0
    self.modifier_hearing = 0
    
    return self
end

--------------------------------------------------------------------------------
function Climate:set_intervals(intervals)
    intervals = intervals or ""
    self.intervals = intervals
end

--------------------------------------------------------------------------------
function Climate:set_module_name(module_name)
    module_name = module_name or ""
    self.module_name = module_name
end

--------------------------------------------------------------------------------
function Climate:set_sandbox_prefix(sandbox_prefix)
    sandbox_prefix = sandbox_prefix or ""
    self.sandbox_prefix = sandbox_prefix
end

--------------------------------------------------------------------------------

function calculate_modifier(modifier_value)

    if modifier_value == 13 then
        new_value = 106
    elseif modifier_value == 1 then
        new_value = -92
    else
        new_value = modifier_value 
    end
    return new_value 
end

function Climate:get_sandbox_settings()
    self.enable = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Enable"):getValue() 
    if not self.enable then
        UTILS.LOGGER("!!!!! DISABLED: " .. self.module_name)
        return
    end
    
    -- Get sandbox settings
    for i = 0, self.intervals do
        self.interval_value[i] = {}
        self.interval_value[i]["value"] = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Interval" ..i):getValue()
        self.interval_value[i]["Speed"] = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Effect" ..i.. "_Speed"):getValue()
        self.interval_value[i]["Strength"] = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Effect" ..i.. "_Strength"):getValue()
        self.interval_value[i]["Toughness"] = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Effect" ..i.. "_Toughness"):getValue()
        self.interval_value[i]["Cognition"] = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Effect" ..i.. "_Cognition"):getValue()
        self.interval_value[i]["Memory"] = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Effect" ..i.. "_Memory"):getValue()
        self.interval_value[i]["Sight"] = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Effect" ..i.. "_Sight"):getValue()
        self.interval_value[i]["Hearing"] = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Effect" ..i.. "_Hearing"):getValue()
    end
    
    -- Debug statements
    UTILS.LOGGER("Getting sandbox settings for: " .. self.sandbox_prefix)
    for i = 0, self.intervals do
        UTILS.LOGGER(" Interval" ..i.. ": " .. self.interval_value[i]["value"])
        UTILS.LOGGER(" Speed: " .. self.interval_value[i]["Speed"])
        UTILS.LOGGER(" Strength: " .. self.interval_value[i]["Strength"])
        UTILS.LOGGER(" Toughness: " .. self.interval_value[i]["Toughness"])
        UTILS.LOGGER(" Cognition: " .. self.interval_value[i]["Cognition"])
        UTILS.LOGGER(" Memory: " .. self.interval_value[i]["Memory"])
        UTILS.LOGGER(" Sight: " .. self.interval_value[i]["Sight"])
        UTILS.LOGGER(" Hearing: " .. self.interval_value[i]["Hearing"])
    end
end

--------------------------------------------------------------------------------
function Climate:get_current_interval()
    -- Get the current value of the climate
    return 0
end

--------------------------------------------------------------------------------
function Climate:get_modifier_values()
    -- update the climate's modifier
    self.get_current_interval()
    local interval_num = 0
    for i = 0, self.intervals do
        UTILS.LOGGER("Looking at Interval" ..i)
        if self.get_current_interval() >= self.interval_value[i]["value"] then
            self.modifier_speed =  calculate_modifier(self.interval_value[i]["Speed"]) - CONSTANTS.sandbox_neutral_modifier
            self.modifier_strength = calculate_modifier(self.interval_value[i]["Strength"]) - CONSTANTS.sandbox_neutral_modifier
            self.modifier_toughness = calculate_modifier(self.interval_value[i]["Toughness"]) - CONSTANTS.sandbox_neutral_modifier
            self.modifier_cognition = calculate_modifier(self.interval_value[i]["Cognition"]) - CONSTANTS.sandbox_neutral_modifier
            self.modifier_memory = calculate_modifier(self.interval_value[i]["Memory"]) - CONSTANTS.sandbox_neutral_modifier
            self.modifier_sight = calculate_modifier(self.interval_value[i]["Sight"]) - CONSTANTS.sandbox_neutral_modifier
            self.modifier_hearing = calculate_modifier(self.interval_value[i]["Hearing"]) - CONSTANTS.sandbox_neutral_modifier
            interval_num = i
        end
    end
    UTILS.LOGGER("------------------------------------------------")
    UTILS.LOGGER("Climate " ..self.sandbox_prefix.. " modifier lore")
    UTILS.LOGGER("Interval" ..interval_num)
    --UTILS.LOGGER("Interval" ..interval_num.. "       : " ..self.interval_value[i]["value"])
    --UTILS.LOGGER("Interval Value  : " ..self.interval_value[interval_num]["value"])
    UTILS.LOGGER("Current Interval: " ..self.get_current_interval())
    UTILS.LOGGER("Speed           : " ..self.modifier_speed )
    UTILS.LOGGER("Strength        : " ..self.modifier_strength )
    UTILS.LOGGER("Toughness       : " ..self.modifier_toughness )
    UTILS.LOGGER("Cognition       : " ..self.modifier_cognition )
    UTILS.LOGGER("Memory          : " ..self.modifier_memory )
    UTILS.LOGGER("Sight           : " ..self.modifier_sight )
    UTILS.LOGGER("Hearing         : " ..self.modifier_hearing )
end


return Climate