----------------------
-- GLOBAL VARIABLES --
----------------------
--[[
How does DZ update the ZOMBIES
get_zombie_lore  > get_latest_lore  
get_climate_lore
--]]

-- Import libraries
local UTILS = require("DZ_Utils")
local CONSTANTS = require("Constants")

-- Zombie variables
local DZ_ZOMBIE_MODULES = {} -- The list of DZ zombie modules which are loaded
local DZ_ZOMBIE_RATIO = {} -- The chance a zombie is assigned to the specific zombie type
local DZ_ZOMBIE_RATIO_TOTAL = 0 
local DZ_CURRENT_ZOMBIE_LORE = {} -- The expected lore of the zombie currently after accounting for the Climate Lore
local DZ_ACTIVE_ZOMBIE_LORE = {} -- The list of zombie lore which DZ will manage

-- Climate variables
local DZ_CLIMATE_MODULES = {} -- The list of DZ climate modules which are loaded
local DZ_CLIMATE_MODIFIERS = {} -- WIP, 


-------------------- ON LOAD FUNCTIONS --------------------
local function DZ_initialising_message()
    UTILS.LOGGER("")
    UTILS.LOGGER("")
    UTILS.LOGGER("--------------------------------------------")
    UTILS.LOGGER("----- INITIALISING DEFINITIVE ZOMBIES  -----")
    UTILS.LOGGER("--------------------------------------------")
end

local function DZ_initialised_message()
    UTILS.LOGGER("--------------------------------------------")
    UTILS.LOGGER("-----  DEFINITIVE ZOMBIES INITIALISED  -----")
    UTILS.LOGGER("--------------------------------------------")
    UTILS.LOGGER("")
    UTILS.LOGGER("")
end

local function DZ_get_modules(module_type)
    --[[ Get the list of zombies that were created for 
         Definitive Zombies from Constants library
    --]]
    UTILS.LOGGER("")
    UTILS.LOGGER("----- GET MODULES:" ..module_type.. "-----")
    
    local dz_modules = {}
    if module_type=="zombies" then
        for k, v in pairs(CONSTANTS.zombies) do
            dz_modules[k] = v
        end
    elseif module_type=="climates" then
        for k, v in pairs(CONSTANTS.climates) do
            dz_modules[k] = v
        end
    end
    
    -- Load a list of possible zombie modules from Constants
    for index, value in ipairs(dz_modules) do
        local dz_module = require(value)
        if dz_module then
            UTILS.LOGGER("Loading: " ..value)
            dz_module:get_sandbox_settings()
            if dz_module.enable then 
                if module_type=="zombies" then
                    table.insert(DZ_ZOMBIE_MODULES, dz_module)
                elseif module_type=="climates" then
                    table.insert(DZ_CLIMATE_MODULES, dz_module)
                end
            end
        end
    end
    UTILS.LOGGER("GET MODULES DONE!")
end

local function DZ_calculate_zombie_ratio()
    --[[ After gathering a list of all DZ zombie modules, 
         Figure out the ratio of all the zombie types
    --]]
    UTILS.LOGGER("")
    UTILS.LOGGER("----- CALCULATING ZOMBIE RATIOS -----")
    for index, zombie_module in ipairs(DZ_ZOMBIE_MODULES) do
        DZ_ZOMBIE_RATIO[index] = tonumber(zombie_module.chance)
        DZ_ZOMBIE_RATIO_TOTAL = DZ_ZOMBIE_RATIO_TOTAL + DZ_ZOMBIE_RATIO[index]
        if index > 1 then
            DZ_ZOMBIE_RATIO[index] = zombie_module.chance + DZ_ZOMBIE_RATIO[index-1] 
        end
        UTILS.LOGGER(zombie_module.module_name.. " chance: " ..DZ_ZOMBIE_RATIO[index])
    end
    UTILS.LOGGER("Total zombie chance: " ..DZ_ZOMBIE_RATIO_TOTAL)
end


-------------------------------------------------------------------------------
------------------------------ RUNTIME FUNCTIONS ------------------------------
-------------------------------------------------------------------------------
local function DZ_update_zombie_lore(baseline_lore, modifier, baseline_minMax)
    new_lore = math.max(baseline_minMax[2], math.min(baseline_lore + DZ_CLIMATE_MODIFIERS[modifier], baseline_minMax[1]))
    UTILS.LOGGER("Updating" .. modifier.. ": " ..new_lore)
    UTILS.LOGGER("baseline: " .. baseline_lore)
    UTILS.LOGGER("modifier: " .. DZ_CLIMATE_MODIFIERS[modifier])
    UTILS.LOGGER("new lore: " .. new_lore)
    return new_lore
end

local function DZ_get_latest_modifiers()
    --[[ Calculate each of the zombie type's lore based on the climate modifiers
    ]]--

    UTILS.LOGGER("-----  DEFINITIVE ZOMBIES MODIFIERS UPDATE  -----")
    -- Get the latest climate modifiers
    DZ_CLIMATE_MODIFIERS["Speed"] = 0
    DZ_CLIMATE_MODIFIERS["Strength"] = 0
    DZ_CLIMATE_MODIFIERS["Toughness"] = 0
    DZ_CLIMATE_MODIFIERS["Cognition"] = 0
    DZ_CLIMATE_MODIFIERS["Memory"] = 0
    DZ_CLIMATE_MODIFIERS["Sight"] = 0
    DZ_CLIMATE_MODIFIERS["Hearing"] = 0
    
    UTILS.LOGGER("GET CLIMATE MODIFIERS")
    for index, climate_module in ipairs(DZ_CLIMATE_MODULES) do
        climate_module:get_modifier_values()
        DZ_CLIMATE_MODIFIERS["Speed"] = DZ_CLIMATE_MODIFIERS["Speed"] + climate_module.modifier_speed
        DZ_CLIMATE_MODIFIERS["Strength"] = DZ_CLIMATE_MODIFIERS["Strength"] + climate_module.modifier_strength 
        DZ_CLIMATE_MODIFIERS["Toughness"] = DZ_CLIMATE_MODIFIERS["Toughness"] + climate_module.modifier_toughness 
        DZ_CLIMATE_MODIFIERS["Cognition"] = DZ_CLIMATE_MODIFIERS["Cognition"] + climate_module.modifier_cognition 
        DZ_CLIMATE_MODIFIERS["Memory"] = DZ_CLIMATE_MODIFIERS["Memory"] + climate_module.modifier_memory 
        DZ_CLIMATE_MODIFIERS["Sight"] = DZ_CLIMATE_MODIFIERS["Sight"] + climate_module.modifier_sight
        DZ_CLIMATE_MODIFIERS["Hearing"] = DZ_CLIMATE_MODIFIERS["Hearing"] + climate_module.modifier_hearing 
    end
    
    -- Calculate the zombies current lore based on the climate modifiers
    UTILS.LOGGER("")
    UTILS.LOGGER("UPDATE ZOMBIE MODIFIERS")
    UTILS.LOGGER("------------------------------------------------")
    for index, zombie_module in ipairs(DZ_ZOMBIE_MODULES) do
        --zombie_module:get_sandbox_settings()
        UTILS.LOGGER("Updating module: " .. zombie_module.module_name)
        DZ_CURRENT_ZOMBIE_LORE[index] = {Speed=DZ_update_zombie_lore(zombie_module.baseline_speed, "Speed", zombie_module.baseline_speed_minMax),
                                         Strength=DZ_update_zombie_lore(zombie_module.baseline_strength, "Strength", zombie_module.baseline_strength_minMax),
                                         Toughness=DZ_update_zombie_lore(zombie_module.baseline_toughness, "Toughness", zombie_module.baseline_toughness_minMax), 
                                         Cognition=DZ_update_zombie_lore(zombie_module.baseline_cognition, "Cognition", zombie_module.baseline_cognition_minMax),
                                         Memory=DZ_update_zombie_lore(zombie_module.baseline_memory, "Memory", zombie_module.baseline_memory_minMax),
                                         Sight=DZ_update_zombie_lore(zombie_module.baseline_sight, "Sight", zombie_module.baseline_sight_minMax),
                                         Hearing=DZ_update_zombie_lore(zombie_module.baseline_hearing, "Hearing", zombie_module.baseline_hearing_minMax),
                                        }
    end
end

local function DZ_get_zombie_type()
    -- Assign a zombie a type from one of the loaded DZ zombie modules
    random_value = ZombRand(0, DZ_ZOMBIE_RATIO_TOTAL)
    for i = 1, #DZ_ZOMBIE_RATIO do
        if random_value < DZ_ZOMBIE_RATIO[i] then
            UTILS.LOGGER("Assigning zombie type: " ..i)
            return i
        end
    end
end

local function DZ_update_sandbox_zombie_type(zombie)
    local zombieData = zombie:getModData()
    for index, zombie_lore in ipairs(DZ_ACTIVE_ZOMBIE_LORE) do
        getSandboxOptions():set("ZombieLore." .. zombie_lore, DZ_CURRENT_ZOMBIE_LORE[zombieData.zombieType][zombie_lore])
    end
end

local function DZ_refresh_zombie(zombie)
    --[[ Checks if a zombie needs to be updated. In multiplayer, toggling zombie:makeInactive() makesthe zombie stop and start     
    --]]
	local zombie_outdated = true
	
    --[[
	if zombie.Speed ~= ZOMBIE_STATE.zombieSandboxSetting.speed or zombie.Toughness ~= ZOMBIE_STATE.zombieSandboxSetting.toughness
	or zombie.Strength ~= ZOMBIE_STATE.zombieSandboxSetting.strength or zombie.Hearing ~= ZOMBIE_STATE.zombieSandboxSetting.hearing
	or zombie.Sight ~= ZOMBIE_STATE.zombieSandboxSetting.sight or zombie.Memory ~= ZOMBIE_STATE.zombieSandboxSetting.memory
	or zombie.Cognition ~= ZOMBIE_STATE.zombieSandboxSetting.cognition
	then
		zombie_outdated = true;
	end
    --]]

    -- If the zombie is outdated, refresh the zombie
    if zombie_outdated then
        DZ_update_sandbox_zombie_type(zombie)
        zombie:makeInactive(true)
        zombie:makeInactive(false)
    end

end

local function DZ_update_zombies(zombie)
    local zombieData = zombie:getModData()
    -- initialise the zombie metadata if they have not been initialised yet
    zombieData.Ticks = zombieData.Ticks or 0
    zombieData.ForceUpdate = zombieData.ForceUpdate or CONSTANTS.reroll_attempts
    zombieData.zombieType = zombieData.zombieType or DZ_get_zombie_type()
    zombieData.Ticks = zombieData.Ticks + 1;
    
    if zombieData.Ticks >= CONSTANTS.update_interval then
        if ZombRand(0, zombieData.ForceUpdate) == 0 then
            if ((not isClient() and not isServer()) or (isClient() and not zombie:isRemoteZombie())) then
                DZ_refresh_zombie(zombie)
                zombieData.Ticks = 0
            end
            -- reset the forceupdate back to the sandbox
            zombieData.ForceUpdate = CONSTANTS.reroll_attempts
        else
            zombieData.ForceUpdate = zombieData.ForceUpdate - 1
            zombieData.Ticks = 0;
        end
    end
end


local function DZ_on_game_time_loaded()
    -- initialise the Constants library
    CONSTANTS.initialise()
    
    -- START
    DZ_initialising_message()
    
    -- Check that at least one zombie and one climate module is activated
    --UTILS.validate_modules();
    
    -- Get a list of zombie lore which DZ will manage
    DZ_ACTIVE_ZOMBIE_LORE = UTILS.get_active_lore()
    
    -- Get DZ modules
    DZ_get_modules("zombies")
    DZ_get_modules("climates")
    
    -- Run a sanity check to ensure that at least one zombie and one climate module is loaded
    
    -- Initialise the modules by getting the lore from the sandbox
    --DZ_get_zombie_lore()
    
    -- Initialise all available climate modules
    
    -- After grabbing all relevant sandbox data, start to do some pre-calculations
    -- before we run DZ
    DZ_calculate_zombie_ratio()
    --DZ_get_latest_modifiers()
    
    -- Run DZ
    Events.EveryTenMinutes.Add(DZ_get_latest_modifiers);
    Events.OnZombieUpdate.Add(DZ_update_zombies);
    
    -- END
    DZ_initialised_message()
end

Events.OnGameTimeLoaded.Add(DZ_on_game_time_loaded);


-- ================================================================================
--[[ TODO
    -- Create a class for zombies
    -- Create a class for climate
    
    --for index, value in ipairs(active_lore) do
    --    UTILS.LOGGER(index, value)
    --end
    --Events.EveryTenMinutes.Add(DZ_update_managers);
    --Events.OnZombieUpdate.Add(DZ_update_zombies);
]]