local UTILS = require("DZ_Utils")

local Zombie = {module_name = "ZombieLore"}
Zombie.__index = Zombie

function Zombie:new()
    -- Keeps track and modifies the sandbox settings for the zombies
    local self = setmetatable({}, Zombie)
    
    self.enable = false
    self.module_name = "Project Zomboid Zombies" -- This is the human readable name for the module
    self.sandbox_prefix = "ZombieLore" -- This is the prefix used to query zombie lore from the sandbox settings
    
    self.normalised_chance = 100
    
    self.baseline_speed = 2
    self.baseline_strength = 2
    self.baseline_toughness = 2
    self.baseline_cognition = 2
    self.baseline_memory = 2
    self.baseline_sight = 2
    self.baseline_hearing = 2
    
    -- The minimum and maximum values of zombie lore
    self.baseline_speed_minMax = {3,1}
    self.baseline_strength_minMax = {3,1}
    self.baseline_toughness_minMax = {3,1}
    self.baseline_cognition_minMax = {3,1}
    self.baseline_memory_minMax = {4,1}
    self.baseline_sight_minMax = {3,1}
    self.baseline_hearing_minMax = {3,1}
    
    return self
end


function Zombie:set_module_name(module_name)
    module_name = module_name or ""
    self.module_name = module_name
end

function Zombie:set_sandbox_prefix(sandbox_prefix)
    sandbox_prefix = sandbox_prefix or ""
    self.sandbox_prefix = sandbox_prefix
end


function Zombie:get_sandbox_settings()
    self.enable = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".EnableZombie"):getValue() 
    self.name = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Name"):getValue()
    self.baseline_speed = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Speed"):getValue()
    self.baseline_strength = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Strength"):getValue()
    self.baseline_toughness = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Toughness"):getValue()
    self.baseline_cognition = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Cognition"):getValue()
    self.baseline_memory = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Memory"):getValue()
    self.baseline_sight = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Sight"):getValue()
    self.baseline_hearing = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Hearing"):getValue()
    self.chance = getSandboxOptions():getOptionByName(self.sandbox_prefix .. ".Chance"):getValue() 
    
    
    -- Debug statements
    if self.enable then
        UTILS.LOGGER("Getting sandbox settings for: " .. self.name)
        UTILS.LOGGER(self.module_name .. " Speed: " .. self.baseline_speed)
        UTILS.LOGGER(self.module_name .. " Strength: " .. self.baseline_strength)
        UTILS.LOGGER(self.module_name .. " Toughness: " .. self.baseline_toughness)
        UTILS.LOGGER(self.module_name .. " Cognition: " .. self.baseline_cognition)
        UTILS.LOGGER(self.module_name .. " Memory: " .. self.baseline_memory)
        UTILS.LOGGER(self.module_name .. " Sight: " .. self.baseline_sight)
        UTILS.LOGGER(self.module_name .. " Hearing: " .. self.baseline_hearing)
        UTILS.LOGGER(self.module_name .. " Chance: " .. self.chance)
    else
        UTILS.LOGGER("!!!!! DISABLED: " .. self.name)
    end
end

--[[
function Zombie:get_chance()
    self.normalised_chance = getSandboxOptions():getOptionByName(self.module_name .. ".Chance"):getValue()

    -- Debug statements
    UTILS.LOGGER("Getting baseline lore for: " .. self.module_name)
    UTILS.LOGGER(self.module_name .. " Chance: " .. self.normalised_chance)

end
--]]
return Zombie